@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016, Stephan Mueller <smueller@chronox.de>
+ * Copyright (C) 2016 - 2017, Stephan Mueller <smueller@chronox.de>
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,7 +56,7 @@
 			* functional enhancements only, consumer
 			* can be left unchanged if enhancements are
 			* not considered. */
-#define PATCHLEVEL 0   /* API / ABI compatible, no functional
+#define PATCHLEVEL 2   /* API / ABI compatible, no functional
 			* changes, no enhancements, bug fixes
 			* only. */
 
@@ -284,10 +284,8 @@ static int drng_seedsource_alloc(struct seed_source *source)
 {
 	int ret = jent_entropy_init();
 
-	if (ret) {
-		printf("The initialization failed with error code %d\n", ret);
+	if (ret)
 		return ret;
-	}
 
 	source->ec = jent_entropy_collector_alloc(0, 0);
 	if (!source->ec)
@@ -377,10 +375,10 @@ static inline void drng_chacha20_update(struct chacha20_state *chacha20)
 	uint32_t i, tmp[CHACHA20_BLOCK_SIZE_WORDS];
 
 	chacha20_block(&chacha20->constants[0], tmp);
-	for (i = 0; i < CHACHA20_KEY_SIZE_WORDS; i++)
+	for (i = 0; i < CHACHA20_KEY_SIZE_WORDS; i++) {
 		chacha20->key.u[i] ^= tmp[i];
-	for (i = 0; i < CHACHA20_KEY_SIZE_WORDS; i++)
 		chacha20->key.u[i] ^= tmp[i + CHACHA20_KEY_SIZE_WORDS];
+	}
 	memset_secure(tmp, 0, sizeof(tmp));
 
 	/* Deterministic increment of nonce as required in RFC 7539 chapter 4 */
@@ -464,34 +462,37 @@ static int drng_chacha20_generate(struct chacha20_state *chacha20,
 static int drng_chacha20_alloc(struct chacha20_drng **out)
 {
 	struct chacha20_drng *drng;
-	uint32_t v = 0;
+	uint32_t i, v = 0;
 	int ret;
 
 	if (drng_chacha20_selftest()) {
-		printf("Selftest failed\n");
 		return -EFAULT;
 	}
 
 	ret = posix_memalign((void *)&drng, CHACHA20_DRNG_ALIGNMENT,
 			     sizeof(*drng));
 	if (ret) {
-		printf("Could not allocate buffer for ChaCha20 state: %d\n",
-		       ret);
 		return -ret;
 	}
 
 	/* prevent paging out of the memory state to swap space */
-	mlock(drng, sizeof(*drng));
+	ret = mlock(drng, sizeof(*drng));
+	if (ret && errno != EPERM && errno != EAGAIN)
+		return -errno;
 
 	memset(drng, 0, sizeof(*drng));
 
 	memcpy(&drng->chacha20.constants[0], "expand 32-byte k", 16);
-	get_time(NULL, &v);
-	drng->chacha20.nonce[0] ^= v;
-	get_time(NULL, &v);
-	drng->chacha20.nonce[1] ^= v;
-	get_time(NULL, &v);
-	drng->chacha20.nonce[2] ^= v;
+
+	for (i = 0; i < CHACHA20_KEY_SIZE_WORDS; i++) {
+		get_time(NULL, &v);
+		drng->chacha20.key.u[i] ^= v;
+	}
+
+	for (i = 0; i < 3; i++) {
+		get_time(NULL, &v);
+		drng->chacha20.nonce[i] ^= v;
+	}
 
 	*out = drng;
 
@@ -513,10 +514,8 @@ int drng_chacha20_reseed(struct chacha20_drng *drng, const uint8_t *inbuf,
 	int ret;
 
 	ret = drng_get_seed(&drng->source, seed, sizeof(seed));
-	if (ret != sizeof(seed)) {
-		printf("Unexpected return code from seed source: %d\n", ret);
+	if (ret != sizeof(seed))
 		return ret;
-	}
 
 	ret = drng_chacha20_seed(&drng->chacha20, seed, sizeof(seed));
 	memset_secure(seed, 0, sizeof(seed));
